#pragma once
#include <iostream>
#include <iomanip>
#include <fstream>
#include <ctime>
#include <vector>
#include <algorithm>
#include <iterator>
#include <string>
#include <cmath>

#define N 100

bool createData(const std::string& fileName) {
	srand(time(nullptr));
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	for (int i = 0; i < N; ++i) {
		fout << rand() % 101 - 50 << "\n";
	}
	fout.close();
	return true;
}


template<template <typename> class container,
			typename type >                 //!!! Передавать нужно тип контейнера, а не его шаблон. 
			                                //!!! Тип хранимых данных можно определить с помощью value_type 
bool createDataGenerate(const std::string& fileName) {
	srand(time(nullptr));
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	container<type> values(N);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	std::generate(values.begin(), values.end(), []() { return (std::rand() % 101 - 50); });
	std::copy(values.begin(), values.end(), std::ostream_iterator<int>(fout, "\n"));
	fout.close();
	return true;
}

template<template <typename> class container,
	typename type >                 //!!! Передавать нужно тип контейнера, а не его шаблон. 
			                                //!!! Тип хранимых данных можно определить с помощью value_type 
	container<type>loadData (const std::string& file)
{
	container<type> result;
	std::ifstream fin(file);

	if (!fin.is_open())
	{
		throw std::exception("Wrong (file name is not valid)!");
	}
	int readBuf;
	while (fin >> readBuf)
	{
		result.push_back(readBuf);
	}
	fin.close();

	return result;
};
template<template <typename> class container,
	typename type > //!!! Везде аналогично
	container<type> loadDataIterator(const std::string& fileName) {
	container<type> result(N);
	std::ifstream fin(fileName);
	if (fin) {
		std::copy(std::istream_iterator<type>(fin), std::istream_iterator<type>(), result.begin());
	}
	return result;
}
//  9
template<template <typename> class container,
	typename type >
bool Modify1(container<type>&cont) {
	if (cont.size() == 0) {
		return false;
	}
	type firstUneven = type();
	type sum = type();
	for (const auto& i : cont) {
		if (i % 2 != 0) {
			firstUneven = i;
			break;
		}
	}
	type sum2 = type();
	for (auto& i : cont) {
		i += firstUneven;
		sum+=i;
		sum2 += std::abs(i);
	}

	cont.push_back(sum);
	cont.push_back(sum2/100);
	return true;
}

template<template <typename> class container,
	typename type >
bool Modify2(type beg, type end) {
	if (end == beg) {
		return false;
	}
	typename type::value_type firstUneven = type();
	typename type:: value_type sum = type();
	for (const auto& iter = beg; iter != end; ++iter) {
		if ((*iter) % 2 != 0) {
			firstUneven = *iter;
			break;
		}
	}

	for (auto& iter = beg; iter != end; ++iter) {
		*iter += firstUneven;
	}

	return true;
}

template<template <typename> class container,
	typename type >
bool Modify3(container<type>& cont) {
	if (cont.end() == cont.begin()) {
		return false;
	}

	auto firstUneven = std::find_if(cont.begin(), cont.end(), [](type i) { return i % 2 != 0; });
	if (!firstUneven) *firstUneven = type();
	std::for_each(cont.begin(), cont.end(), [firstUneven](auto& obj) { obj += *firstUneven; });

	return true;
}

template<template <typename> class container,
	typename type >
bool Modify4(const std::string& fileName) {

	container<type> buffer(N);
	std::ifstream fin(fileName);
	if (!fin) {
	throw std::exception("Wrong (file name is not valid)!");
	}
	std::copy(std::istream_iterator<type>(fin), std::istream_iterator<type>(), buffer.begin());
	auto firstUneven = std::find_if(buffer.begin(), buffer.end(), [](type i) { return i % 2 != 0; });
	std::ofstream fout("res.txt");
	std::transform(buffer.begin(), buffer.end(), std::ostream_iterator<type>(fout, "\n"),
		[ firstUneven](auto& obj) { return obj += *firstUneven; });

	return true;
}

template<template <typename> class container,
	typename type >
bool outputResult(const std::string& fileName, const container<type>& cont) {
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	for (int i = 0; i < cont.size(); ++i) {
		fout << cont[i] <<"\n";
	}
	fout.close();
	return true;
}
template<template <typename> class container,
	typename type >
bool outputResultCopy(const std::string& fileName, const container<type>& cont) {
	srand(time(nullptr));
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	std::copy(cont.begin(), cont.end(), std::ostream_iterator<type>(fout, "\n"));
	fout.close();
	return true;
}
